/*
 * *********************************************************************************************************************
 *
 * TheseFoolishThings: Miscellaneous utilities
 * http://tidalwave.it/projects/thesefoolishthings/modules/it-tidalwave-messagebus
 *
 * Copyright (C) 2009 - 2021 by Tidalwave s.a.s. (http://tidalwave.it)
 *
 * *********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 * *********************************************************************************************************************
 *
 * git clone https://bitbucket.org/tidalwave/thesefoolishthings-src
 * git clone https://github.com/tidalwave-it/thesefoolishthings-src
 *
 * *********************************************************************************************************************
 */
package it.tidalwave.messagebus.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import javax.annotation.Nonnull;

/***********************************************************************************************************************
 *
 * Designates a class as a simple subscriber of a message bus. This makes it possible to avoid plumbing code for
 * subscribing to topics and use instead {@link ListensTo} with {@link it.tidalwave.messagebus.MessageBusHelper}.
 * An optional {@link #source()} parameter can be specified to choose a message bus for systems where more than one are
 * available.
 *
 * @author  Fabrizio Giudici
 *
 **********************************************************************************************************************/
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.TYPE)
@Documented
public @interface SimpleMessageSubscriber
  {
    public static final String DEFAULT_SOURCE = "applicationMessageBus";

    /*******************************************************************************************************************
     *
     * The name of the source that this annotation refers to.
     *
     * @return  the source name
     *
     ******************************************************************************************************************/
    @Nonnull
    String source() default DEFAULT_SOURCE;
  }
